# -*- coding: utf-8 -*-

from django.template import Library, Variable
from django.conf import settings
from django.utils import simplejson
from django.utils.translation import ugettext_lazy
from django.utils.translation import get_language
from django.contrib.contenttypes.models import ContentType

from inplaceeditform.commons import (get_form_class, apply_filters,
                                     special_procesing, 
                                     has_translation)

register = Library()

def inplace_media(context):
    return context.update({
            'user': context['request'].user,
            'MEDIA_URL': context.get('MEDIA_URL', settings.MEDIA_URL),
            'ADMIN_MEDIA_PREFIX': settings.ADMIN_MEDIA_PREFIX,
    })
register.inclusion_tag("inplace_media.html", takes_context=True)(inplace_media)


def inplace_edit(context, obj, expression, form='', expression2=None):
    content_type_id = ContentType.objects.get_for_model(obj.__class__).id
    form_class = get_form_class(form, content_type_id)
    prefix = '%s_%s'%(content_type_id, obj.id)
    form_obj = form_class(instance=obj, prefix=prefix)
    tokens = expression.split('|')
    field, filters = tokens[0], tokens[1:]
    current_language = get_language()
    field_lang = None
    if field in form_obj.fields:
        field_obj = form_obj[field]
    else:
        field_lang = "%s_%s" %(field, current_language)
        field_obj = form_obj[field_lang]


    value = getattr(obj, field, '-----')
    value = special_procesing(field_obj, value)


    value = apply_filters(value, filters)

    empty_value = (isinstance(value, str) and value.strip() == u'' or not value)

    if field_lang and not has_translation(field_obj, obj, current_language):
        missing_msg = u'<h3 class="missing-translation">%s</h3>' % ugettext_lazy('Translation missing')
        form_obj.initial[field_obj.name] = missing_msg 
        old_value = form_obj.initial[field_obj.name]
        if old_value:
            form_obj.initial[field_obj.name] = old_value 


    if expression2:
        tokens = expression2.split('|')
        field2, filters2 = tokens[0], tokens[1:]
        value2 = Variable(field2).resolve(context)
        value2 = apply_filters(value2, filters2)
    else:
        value2 = None

    user = context.get('user',None)
    can_edit_this_object = getattr(obj, 'can_edit_this_object', None)
    if can_edit_this_object:
        can_edit_this_object = can_edit_this_object(user)
    can_edit_this_object = can_edit_this_object or  (user.is_authenticated() and user.is_staff)
    return {
            'obj': obj,
            'field': field_obj,
            'content_type_id': content_type_id,
            'form': form,
            'form_obj': form_obj,
            'value':  value,
            'value2':  value2,
            'empty_value': empty_value,
            'filters': simplejson.dumps(filters),
            'MEDIA_URL': context.get('MEDIA_URL',''),
            'user': user,
            'can_edit_this_object': can_edit_this_object,
    }
register.inclusion_tag("inplace_edit.html", takes_context=True)(inplace_edit)




# -*- coding: utf-8 -*-
from django import template
from django.forms.models import modelform_factory, ModelMultipleChoiceField
from django.forms.fields import MultipleChoiceField
from django.contrib.contenttypes.models import ContentType

has_transmeta = False
DEFAULT_VALUE = '<h3 class="missing-translation">Translation missing</h3>'
try:
    from transmeta import TransMeta
    has_transmeta = True
except ImportError:
    pass

def change_foreing_key(obj):
    obj_dict = obj.__dict__
    obj_dict_result = obj_dict.copy()
    for key, value in obj_dict.items():
        if '_id' in key:
            key2 = key.replace('_id','')
            obj_dict_result[key2] = obj_dict_result[key]
            del obj_dict_result[key]

    manytomany_list = obj._meta.many_to_many
    for manytomany in manytomany_list:
        obj_dict_result[manytomany.name] = [ obj_rel.id for obj_rel in manytomany.value_from_object(obj)]
    return obj_dict_result


def apply_filters(value, filters):
    if value and filters:
        filters_str = '|%s' % '|'.join(filters)
        value = template.Template("""{{ value%s }}""" % filters_str).render(template.Context({'value': value}))
    return value


def get_form_class(form_str=None, content_type_id=None):
    if form_str:
        path_split = form_str.split('.')
        return getattr(__import__('.'.join(path_split[:len(path_split)-1]), {}, {}, [path_split[len(path_split)-1]]), path_split[(len(path_split)-1)])
    contenttype = ContentType.objects.get(id=content_type_id)
    return modelform_factory(contenttype.model_class())


def special_procesing(field_obj, value):
    if isinstance(field_obj.field, MultipleChoiceField) or isinstance(field_obj.field, ModelMultipleChoiceField):
        value = value.all()
        value =[v.__unicode__() for v in value]
        value = ','.join(value)

    if field_obj.form._meta.model._meta.get_field(field_obj.name) and field_obj.form._meta.model._meta.get_field(field_obj.name).choices:
        choices_dict = dict(field_obj.form._meta.model._meta.get_field(field_obj.name).choices)
        value = choices_dict[value]

    return value


def has_translation(field_obj, obj, current_language):
    if has_transmeta and getattr(obj._meta, 'translatable_fields', None):
        name_field = field_obj.name.split('_')
        if len(name_field) > 1 and name_field[0] in obj._meta.translatable_fields and \
           getattr(obj, field_obj.name) and not getattr(obj, field_obj.name) == DEFAULT_VALUE:
            return True
    return False

def transmeta_procesing(field_obj, value, obj, current_language):
    if has_transmeta and not has_translation(field_obj, obj, current_language):
        field_name_generic = field_obj.name.split('_')
        if len(field_name_generic) > 1:
            value = getattr(obj, field_name_generic[0])
    return value


class InplaceEditModel(object):

    def can_edit_this_object(self, user):
        return False
    
    @property
    def text_empty_value(self):
        return ''
    
    @property
    def can_be_value_none(self):
        return True



# -*- coding: utf-8 -*-
from decimal import Decimal

from django.conf import settings
from django.core.serializers.json import DateTimeAwareJSONEncoder
from django.db import models
from django.forms.models import ModelMultipleChoiceField
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import simplejson
from django.contrib.contenttypes.models import ContentType

from commons import (get_form_class, change_foreing_key, apply_filters,
                     special_procesing, transmeta_procesing)


def save_ajax(request):
    if request.POST:
        try:
            post = request.POST.copy()
            application_form = post.get('application_form')
            form = post.get('form', None)
            content_type_id = post.get('content_type_id')
            obj_id = post.get('id')
            field = post.get('field')
            value = simplejson.loads(post.get('value'))
            filters = simplejson.loads(post.get('filters','[]'))
            is_cmsutils = post.get('is_cmsutils', False)
            form_class = get_form_class(form, content_type_id)
            model_class = form_class._meta.model
            contenttype = ContentType.objects.get(id=content_type_id)
            assert contenttype.model_class() is model_class # check for hacker attack by changing POST
            obj = get_object_or_404(model_class,id=obj_id)
            user = request.user
            if not user.is_staff and not user.is_superuser and getattr(obj, 'can_edit_this_object', None) and not getattr(obj, 'can_edit_this_object')(user):
                json_dict = simplejson.dumps({'errors': True})
            else:
                new_data = change_foreing_key(obj)
                if isinstance(form_class.base_fields[field], ModelMultipleChoiceField):
                    new_data[field] = value.split(',')
                else:
                    new_data[field] = value

                form_obj = form_class(data=new_data, instance=obj)
                if form_obj.is_valid():
                    current_language = request.session.get('django_language',
                                                        settings.LANGUAGE_CODE)
                    obj = form_obj.save(current_language)
                    value = getattr(obj, field)
                    value = special_procesing(form_obj[field], value)
                    value = transmeta_procesing(form_obj[field], value, obj,
                                            current_language)
                    value = apply_filters(value, filters)
                    if not value and not getattr(obj, 'can_be_value_none', True):
                        value = obj.text_empty_value
                    json_dict = json_encode({'errors':False, 'value': value})
                else:
                    python_dict = dict(form_obj.errors)
                    python_dict['errors'] = True
                    json_dict = json_encode(dict(python_dict))
        except: # very scary!!!!
            json_dict = simplejson.dumps({'errors': True})

        return HttpResponse(json_dict,mimetype='application/json')

def json_encode(data, ensure_ascii=False):
    """
    The main issues with django's default json serializer is that properties that
    had been added to a object dynamically are being ignored (and it also has 
    problems with some models).
    """
    def _any(data):
        ret = None
        if isinstance(data, list):
            ret = _list(data)
        elif isinstance(data, dict):
            ret = _dict(data)
        elif isinstance(data, Decimal):
            # json.dumps() cant handle Decimal
            ret = str(data)
        elif isinstance(data, models.query.QuerySet):
            # Actually its the same as a list ...
            ret = _list(data)
        elif isinstance(data, models.Model):
            ret = _model(data)
        else:
            try:
                ret = data.__unicode__()
            except:
                ret = data
        return ret

    def _model(data):
        return data.__unicode__()

    def _list(data):
        ret = []
        for v in data:
            ret.append(_any(v))
        return ret

    def _dict(data):
        ret = {}
        for k,v in data.items():
            ret[k] = _any(v)
        return ret

    ret = _any(data)

    return simplejson.dumps(ret, cls=DateTimeAwareJSONEncoder, ensure_ascii=ensure_ascii)